<?php
/**
 * This is the entrypoint for the locrMAPS WordPress plugin.
 *
 * @package locrMAPS
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMaps\Admin;

use Locr\WordPress\Plugin\LocrMaps\Admin\SettingsPage\OnlineMaps;
use Locr\WordPress\Plugin\LocrMaps\DDD\Entities\Icon;
use Locr\WordPress\Plugin\LocrMaps\DDD\Entities\OnlineMap\Location;
use Locr\WordPress\Plugin\LocrMaps\DDD\Repositories\{CountriesAndLanguages, Icons, OnlineMapLocations as OnlineMapLocationsRepository};
use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\{CountryAndLanguage, GeneralOptions, SettingsMessageId};
use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\GeneralOptions\AccessKey;
use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\OnlineMap\Id as OnlineMapId;
use Locr\WordPress\Plugin\LocrMaps\TypedSequence;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for the ONLINEmaps locations list table.
 */
final class OnlineMapLocationsTable extends \WP_List_Table {
	/**
	 * This function initializes the ONLINEmaps locations list table.
	 *
	 * @param OnlineMapId $onlinemap_id The ONLINEmap id.
	 */
	public function __construct( private OnlineMapId $onlinemap_id ) {
		parent::__construct();
	}

	/**
	 * This function displays the checkbox column.
	 *
	 * @param Location $item The item.
	 */
	public function column_cb( $item ): string {
		return sprintf(
			'<input type="checkbox" name="id[]" value="%s" />',
			$item->id->value,
		);
	}

	/**
	 * This function displays the table.
	 *
	 * @param Location $item The item.
	 * @param string   $column_name The column name.
	 * @return mixed
	 */
	public function column_default( $item, $column_name ): mixed {
		switch ( $column_name ) {
			case 'country':
				return $item->address->country;
			case 'state':
				return $item->address->state;
			case 'city':
				return $item->address->city;
			case 'postal':
				return $item->address->postal;
			case 'street':
				return $item->address->street;
			case 'house':
				return $item->address->house;
			case 'icon':
				$icons_repository = new Icons();
				$icon             = $icons_repository->get_by_id( $item->icon_id );
				if ( ! is_null( $icon ) ) {
					return '<img src="' . esc_attr( (string) $icon->url ) . '" style="max-width: 36px; max-height: 36px;" alt="location icon" />';
				}
				return '';
			case 'id':
				return $item->id->value;
			case 'latitude':
				return $item->position->latitude;
			case 'longitude':
				return $item->position->longitude;
			default:
				break;
		}

		return null;
	}

	/**
	 * This function displays the ID column.
	 *
	 * @param Location $item The item.
	 */
	protected function column_id( $item ): string {
		$admin_page_url = admin_url( 'admin.php' );

		$id = $item->id->value;

		$page = filter_input( INPUT_GET, 'page', FILTER_DEFAULT ) ?? filter_input( INPUT_POST, 'page', FILTER_DEFAULT );

		$actions                  = array();
		$query_args_edit          = array(
			'page'     => $page,
			'action'   => 'edit-location',
			'id'       => $id,
			'_wpnonce' => wp_create_nonce( 'edit_onlinemap_locations_item' ),
		);
		$edit_onlinemaps_item_url = esc_url( add_query_arg( $query_args_edit, $admin_page_url ) ) . '#TB_inline?dummy=&width=600&height=625&inlineId=onlinemap-location-overlay-content&location-data=' . esc_attr( $item->to_json() );
		$actions['edit']          = sprintf( '<a href="%s" class="thickbox">%s</a>', $edit_onlinemaps_item_url, __( 'Edit', 'default' ) );

		$query_args_delete = array(
			'page'     => $page,
			'action'   => 'delete-location',
			'id'       => $id,
			'_wpnonce' => wp_create_nonce( 'delete_onlinemap_locations_item' ),
		);
		$delete_url        = esc_url( add_query_arg( $query_args_delete, $admin_page_url ) );
		$actions['trash']  = sprintf( '<a href="%s">%s</a>', $delete_url, __( 'Delete', 'default' ) );

		return sprintf( '%1$s %2$s', $id, $this->row_actions( $actions ) );
	}

	/**
	 * This function builds the location overlay.
	 */
	public function display_location_overlay(): void {
		$general_options = new GeneralOptions( only_option: AccessKey::OPTION_KEY );
		$access_key      = (string) $general_options->access_key;

		print '<div id="onlinemap-location-overlay-content" style="display: none;">';
		print '	<div style="display: flex; flex-direction: column;">';
		print '		<div id="location-notice"></div>';
		print '		<form id="location-address-form">';
		print '			<h2>' . esc_html__( 'Address', 'locrmaps-textdomain' ) . ':</h2>';
		print '			<table id="location-address" style="width: 100%;">';
		print '				<tbody>';
		print '					<tr>';
		print '						<td style="width: 14%;"><label for="location-address-street">' . esc_html__( 'Street', 'locrmaps-textdomain' ) . ':</label></td>';
		print '						<td style="width: 34%;"><input type="text" id="location-address-street" name="location-address-street" value="" /></td>';
		print '						<td style="width: 4%;">&nbsp;</td>';
		print '						<td style="width: 14%;"><label for="location-address-house">' . esc_html__( 'House', 'locrmaps-textdomain' ) . ':</label></td>';
		print '						<td style="width: 34%;"><input type="text" id="location-address-house" name="location-address-house" value="" /></td>';
		print '					</tr>';
		print '					<tr>';
		print '						<td><label for="location-address-city">' . esc_html__( 'City', 'locrmaps-textdomain' ) . ':</label></td>';
		print '						<td><input type="text" id="location-address-city" name="location-address-city" value="" /></td>';
		print '						<td>&nbsp;</td>';
		print '						<td><label for="location-address-postal">' . esc_html__( 'ZIP', 'locrmaps-textdomain' ) . ':</label></td>';
		print '						<td><input type="text" id="location-address-postal" name="location-address-postal" value="" /></td>';
		print '					</tr>';
		print '					<tr>';
		print '						<td><label for="location-address-state">' . esc_html__( 'State', 'locrmaps-textdomain' ) . ':</label></td>';
		print '						<td><input type="text" id="location-address-state" name="location-address-state" value="" /></td>';
		print '						<td>&nbsp;</td>';
		print '						<td><label for="location-address-country">' . esc_html__( 'Country', 'locrmaps-textdomain' ) . ':</label></td>';
		print '						<td>';
		print '							<select id="location-address-country" name="location-address-country" required="required">';
		print '								<option value="" selected="selected">--- ' . esc_html__( 'Please select a country', 'locrmaps-textdomain' ) . ' ---</option>';
		$locale                             = get_locale();
		$countries_and_languages_repository = new CountriesAndLanguages();
		$countries_and_languages            = $countries_and_languages_repository
			->get_all()
			->getOrderedIterator(
				function ( CountryAndLanguage $a, CountryAndLanguage $b ) use ( $locale ) {
					$a_name = $a->try_get_localized_name( $locale )->primary()->value;
					$b_name = $b->try_get_localized_name( $locale )->primary()->value;
					if ( class_exists( '\Normalizer' ) ) {
						$a_name = \Normalizer::normalize( $a_name, \Normalizer::FORM_D );
						$b_name = \Normalizer::normalize( $b_name, \Normalizer::FORM_D );
					}
					return $a_name <=> $b_name;
				}
			);

		$top_countries       = array( 'DEU', 'USA' );
		$top_countries_found = false;
		foreach ( $top_countries as $top_country ) {
			foreach ( $countries_and_languages as $country_and_language ) {
				if ( $country_and_language instanceof CountryAndLanguage && $country_and_language->iso3166alpha3->value === $top_country ) {
					$top_countries_found = true;
					print '								<option' . PHP_EOL;
					print '									value="' . esc_attr( $country_and_language->iso3166alpha3->value ) . '"' . PHP_EOL;
					print '									data-iso3166-alpha2="' . esc_attr( $country_and_language->iso3166alpha2->value ) . '"' . PHP_EOL;
					print '									data-iso3166-numeric="' . esc_attr( (string) $country_and_language->iso3166numeric->value ) . '">' . esc_html( $country_and_language->try_get_localized_name( $locale )->primary()->value ) . '</option>';
				}
			}
		}
		if ( $top_countries_found ) {
			print '								<hr/>';
		}
		foreach ( $countries_and_languages as $country_and_language ) {
			if ( $country_and_language instanceof CountryAndLanguage && ! in_array( $country_and_language->iso3166alpha3->value, $top_countries, true ) ) {
				print '								<option' . PHP_EOL;
				print '									value="' . esc_attr( $country_and_language->iso3166alpha3->value ) . '"' . PHP_EOL;
				print '									data-iso3166-alpha2="' . esc_attr( $country_and_language->iso3166alpha2->value ) . '"' . PHP_EOL;
				print '									data-iso3166-numeric="' . esc_attr( (string) $country_and_language->iso3166numeric->value ) . '">' . esc_html( $country_and_language->try_get_localized_name( $locale )->primary()->value ) . '</option>';
			}
		}
		print '							</select>';
		print '						</td>';
		print '					</tr>';
		print '				</tbody>';
		print '				<tfoot>';
		print '					<tr>';
		print '						<td colspan="4"><button type="submit" id="location-address-search" class="button button-primary">' . esc_html__( 'Search', 'default' ) . '</button></td>';
		print '					</tr>';
		print '				</tfoot>';
		print '			</table>';
		print '		</form>';
		print '		<div style="display: flex; flex-direction: row;">';
		print '			<div style="flex-grow: 10;">';
		print '				<h2>' . esc_html__( 'Location Preview', 'locrmaps-textdomain' ) . ':</h2>';
		print '				<div id="location-preview-map" data-access-key="' . esc_attr( $access_key ) . '" style="width: 100%; height: 300px;"></div>';
		print '			</div>';
		print '			<div style="margin-left: 16px; width: 100px;">';
		print '				<h2>' . esc_html__( 'Icon', 'locrmaps-textdomain' ) . ':</h2>';
		print '				<div style="background-color: #eee; height: 284px; padding: 8px; overflow-y: auto;">';
		$icons_repository = new Icons();
		$icons            = $icons_repository->get_all_icons();
		print '					<table id="location-icons" class="location-icon-table">';
		print '						<tbody>';
		$icon_counter = 0;
		foreach ( $icons as $icon ) {
			if ( $icon instanceof Icon ) {
				++$icon_counter;
				print '							<tr>';
				print '								<td';
				if ( 1 === $icon_counter ) {
					print ' class="selected"';
				}
				$icon_meta = wp_json_encode( $icon->to_array() );
				if ( false === $icon_meta ) {
					$icon_meta = '{}';
				}
				print '><img src="' . esc_attr( (string) $icon->url ) . '" data-meta="' . esc_attr( $icon_meta ) . '" style="max-width: 100%;" /></td>';
				print '							</tr>';
			}
		}
		print '						</tbody>';
		print '					</table>';
		print '				</div>';
		print '			</div>';
		print '		</div>';
		print '		<div style="margin-top: 16px;">';
		print '			<input type="hidden" id="location-id" name="location-id" value=""/>';
		print '			<button id="save-location-icon" class="button button-primary" disabled="disabled">' . esc_html__( 'Save Icon', 'locrmaps-textdomain' ) . '</button>';
		print '		</div>';
		print '	</div>';
		print '</div>';
	}

	/**
	 * This function builds the location overlay button.
	 */
	public function display_location_overlay_button(): void {
		print '<a href="#TB_inline?dummy=&width=600&height=625&inlineId=onlinemap-location-overlay-content" class="thickbox button button-primary" title="' . esc_attr__( 'Location', 'locrmaps-textdomain' ) . '">' . esc_html__( 'Add', 'default' ) . ' ' . esc_html__( 'Add Icon', 'locrmaps-textdomain' ) . '</a>';
	}

	/**
	 * This function displays the icon URL column.
	 *
	 * @return array<mixed>
	 */
	public function get_bulk_actions(): array {
		return array(
			'bulk-delete' => __( 'Delete', 'default' ),
		);
	}

	/**
	 * This function initializes the icon list table.
	 *
	 * @return array<mixed> The columns.
	 */
	public function get_columns(): array {
		return array(
			'cb'        => '<input type="checkbox" />',
			'id'        => __( 'ID', 'locrmaps-textdomain' ),
			'icon'      => __( 'Icon', 'locrmaps-textdomain' ),
			'street'    => __( 'Street', 'locrmaps-textdomain' ),
			'house'     => __( 'House', 'locrmaps-textdomain' ),
			'postal'    => __( 'ZIP', 'locrmaps-textdomain' ),
			'city'      => __( 'City', 'locrmaps-textdomain' ),
			'state'     => __( 'State', 'locrmaps-textdomain' ),
			'country'   => __( 'Country', 'locrmaps-textdomain' ),
			'latitude'  => __( 'Latitude', 'locrmaps-textdomain' ),
			'longitude' => __( 'Longitude', 'locrmaps-textdomain' ),
		);
	}

	/**
	 * This function gets the data.
	 *
	 * @return TypedSequence The data.
	 */
	public function get_data(): TypedSequence {
		$onlinemap_locations_repository = new OnlineMapLocationsRepository();
		return $onlinemap_locations_repository->get_by_id( $this->onlinemap_id );
	}

	/**
	 * This function handles the table actions.
	 *
	 * @param OnlineMapId $onlinemap_id The ONLINEmap ID.
	 */
	public function handle_table_actions( OnlineMapId $onlinemap_id ): void {
		$request_method = filter_input( INPUT_SERVER, 'REQUEST_METHOD', FILTER_DEFAULT ) ?? '';
		if ( empty( $request_method ) && isset( $_SERVER['REQUEST_METHOD'] ) ) {
			$request_method = filter_var( wp_unslash( $_SERVER['REQUEST_METHOD'] ), FILTER_DEFAULT );
		}
		if ( 'POST' !== $request_method ) {
			return;
		}

		$admin_url = admin_url( 'admin.php?page=' . OnlineMaps::SUB_PAGE_NAME );

		$referer = (string) ( filter_input( INPUT_SERVER, 'HTTP_REFERER', FILTER_DEFAULT ) ?? '' );
		if ( empty( $referer ) && isset( $_SERVER['HTTP_REFERER'] ) ) {
			$referer = (string) filter_var( wp_unslash( $_SERVER['HTTP_REFERER'] ), FILTER_DEFAULT );
		}
		if ( ! empty( $referer ) ) {
			$admin_url = $referer;
		}

		$wp_nonce      = null;
		$nonce_request = filter_input( INPUT_POST, '_wpnonce', FILTER_DEFAULT );
		if ( is_string( $nonce_request ) ) {
			$wp_nonce = $nonce_request;
		}

		$action  = filter_input( INPUT_GET, 'action', FILTER_DEFAULT ) ?? filter_input( INPUT_POST, 'action', FILTER_DEFAULT );
		$action2 = filter_input( INPUT_GET, 'action2', FILTER_DEFAULT ) ?? filter_input( INPUT_POST, 'action2', FILTER_DEFAULT );

		if ( 'bulk-delete' === $action || 'bulk-delete' === $action2 ) {
			if ( ! is_string( $wp_nonce ) ) {
				wp_safe_redirect( add_query_arg( 'message', SettingsMessageId::NoNonce->value, $admin_url ) );
				exit;
			}

			$ids = filter_input( INPUT_POST, 'id', FILTER_DEFAULT, FILTER_REQUIRE_ARRAY );
			if ( is_array( $ids ) ) {
				$nonce_action = 'bulk-delete';
				if ( isset( $this->_args['plural'] ) && is_string( $this->_args['plural'] ) ) {
					$nonce_action = 'bulk-' . $this->_args['plural'];
				}
				if ( ! wp_verify_nonce( $wp_nonce, $nonce_action ) ) {
					wp_safe_redirect( add_query_arg( 'message', SettingsMessageId::InvalidNonce->value, $admin_url ) );
					exit;
				}

				$onlinemap_locations_repository = new OnlineMapLocationsRepository();
				try {
					$onlinemap_locations_repository->delete_all_ids( ids: $ids, onlinemap_id: $onlinemap_id );

					wp_safe_redirect( add_query_arg( 'message', SettingsMessageId::OnlineMapLocationsDeleted->value, $admin_url ) );
					exit;
				} catch ( \Exception $exc ) {
					wp_safe_redirect( add_query_arg( 'message', SettingsMessageId::OnlineMapLocationsNotDeleted->value, $admin_url ) );
				}
			}
		}
	}

	/**
	 * This function prepares the items.
	 */
	public function prepare_items(): void {
		$data     = $this->get_data();
		$columns  = $this->get_columns();
		$hidden   = array();
		$sortable = array(
			'id' => array( 'id', true ),
		);

		$this->_column_headers = array( $columns, $hidden, $sortable );
		$this->items           = $data->to_array();
	}
}
