<?php
/**
 * This is the entrypoint for the locrMAPS WordPress plugin.
 *
 * @package locrMAPS
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMaps\Api;

use Locr\WordPress\Plugin\LocrMaps\DDD\Entities\OnlineMap\Location;
use Locr\WordPress\Plugin\LocrMaps\DDD\Repositories\{OnlineMapLocations, OnlineMaps};
use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\Icon\Id as IconId;
use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\OnlineMap\Id as OnlineMapId;
use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\OnlineMap\Location\{Address, Id as LocationId, Position};
use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\OnlineMap\Location\Address\{City, Country, House, State, Street, Postal};

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for the shortcode.
 */
final class OnlineMapLocation {
	/**
	 * This function registers the add location route.
	 */
	public static function register_add_location_route(): void {
		register_rest_route(
			route_namespace: 'locrmaps/onlinemap',
			route: '/(?P<id>[0-9]+)/location',
			args: array(
				'methods'             => \WP_REST_Server::CREATABLE,
				'callback'            => function ( \WP_REST_Request $request ) {
					$id = $request->get_param( 'id' );
					if ( is_null( $id ) ) {
						return new \WP_Error( 'bad_request', 'Missing "id" parameter.', array( 'status' => 400 ) );
					}
					if ( ! is_numeric( $id ) ) {
						return new \WP_Error( 'bad_request', 'Invalid "id" parameter.', array( 'status' => 400 ) );
					}

					$onlinemaps_repository = new OnlineMaps();
					$onlinemap             = $onlinemaps_repository->get_by_id( new OnlineMapId( (int) $id ) );
					if ( is_null( $onlinemap ) ) {
						return new \WP_Error( 'not_found', 'The online map could not be found.', array( 'status' => 404 ) );
					}

					$icon_id = $request->get_param( 'icon_id' );
					if ( is_null( $icon_id ) ) {
						return new \WP_Error( 'bad_request', 'Missing "icon_id" parameter.', array( 'status' => 400 ) );
					}
					if ( ! is_numeric( $icon_id ) ) {
						return new \WP_Error( 'bad_request', 'Invalid "icon_id" parameter.', array( 'status' => 400 ) );
					}

					$latitude = $request->get_param( 'latitude' );
					if ( is_null( $latitude ) ) {
						return new \WP_Error( 'bad_request', 'Missing "latitude" parameter.', array( 'status' => 400 ) );
					}
					if ( ! is_numeric( $latitude ) ) {
						return new \WP_Error( 'bad_request', 'Invalid "latitude" parameter.', array( 'status' => 400 ) );
					}
					$longitude = $request->get_param( 'longitude' );
					if ( is_null( $longitude ) ) {
						return new \WP_Error( 'bad_request', 'Missing "longitude" parameter.', array( 'status' => 400 ) );
					}
					if ( ! is_numeric( $longitude ) ) {
						return new \WP_Error( 'bad_request', 'Invalid "longitude" parameter.', array( 'status' => 400 ) );
					}

					$onlinemap_locations_repository = new OnlineMapLocations();
					$location = new Location(
						id: new LocationId( PHP_INT_MAX ),
						icon_id: new IconId( (int) $icon_id ),
						position: new Position( (float) $latitude, (float) $longitude ),
						address: new Address(
							country: new Country( $request->get_param( 'country' ) ?? '' ),
							state: new State( $request->get_param( 'state' ) ?? '' ),
							city: new City( $request->get_param( 'city' ) ?? '' ),
							postal: new Postal( $request->get_param( 'postal' ) ?? '' ),
							street: new Street( $request->get_param( 'street' ) ?? '' ),
							house: new House( $request->get_param( 'house' ) ?? '' ),
						)
					);
					$onlinemap_locations_repository->add( new OnlineMapId( $id ), $location );

					$response = array(
						'id'       => $id,
						'icon_id'  => $icon_id,
						'position' => array(
							'latitude'  => $latitude,
							'longitude' => $longitude,
						),
					);

					return rest_ensure_response( $response );
				},
				'permission_callback' => '__return_true',
			)
		);
	}

	/**
	 * This function registers the update location route.
	 */
	public static function register_update_location_route(): void {
		register_rest_route(
			route_namespace: 'locrmaps/onlinemap',
			route: '/(?P<id>[0-9]+)/location/(?P<location_id>[0-9]+)',
			args: array(
				'methods'             => \WP_REST_Server::EDITABLE,
				'callback'            => function ( \WP_REST_Request $request ) {
					$id = $request->get_param( 'id' );
					if ( is_null( $id ) ) {
						return new \WP_Error( 'bad_request', 'Missing "id" parameter.', array( 'status' => 400 ) );
					}
					if ( ! is_numeric( $id ) ) {
						return new \WP_Error( 'bad_request', 'Invalid "id" parameter.', array( 'status' => 400 ) );
					}

					$location_id = $request->get_param( 'location_id' );
					if ( is_null( $location_id ) ) {
						return new \WP_Error( 'bad_request', 'Missing "location_id" parameter.', array( 'status' => 400 ) );
					}
					if ( ! is_numeric( $location_id ) ) {
						return new \WP_Error( 'bad_request', 'Invalid "location_id" parameter.', array( 'status' => 400 ) );
					}

					$onlinemaps_repository = new OnlineMaps();
					$onlinemap             = $onlinemaps_repository->get_by_id( new OnlineMapId( (int) $id ) );
					if ( is_null( $onlinemap ) ) {
						return new \WP_Error( 'not_found', 'The online map could not be found.', array( 'status' => 404 ) );
					}

					$icon_id = $request->get_param( 'icon_id' );
					if ( is_null( $icon_id ) ) {
						return new \WP_Error( 'bad_request', 'Missing "icon_id" parameter.', array( 'status' => 400 ) );
					}
					if ( ! is_numeric( $icon_id ) ) {
						return new \WP_Error( 'bad_request', 'Invalid "icon_id" parameter.', array( 'status' => 400 ) );
					}

					$latitude = $request->get_param( 'latitude' );
					if ( is_null( $latitude ) ) {
						return new \WP_Error( 'bad_request', 'Missing "latitude" parameter.', array( 'status' => 400 ) );
					}
					if ( ! is_numeric( $latitude ) ) {
						return new \WP_Error( 'bad_request', 'Invalid "latitude" parameter.', array( 'status' => 400 ) );
					}
					$longitude = $request->get_param( 'longitude' );
					if ( is_null( $longitude ) ) {
						return new \WP_Error( 'bad_request', 'Missing "longitude" parameter.', array( 'status' => 400 ) );
					}
					if ( ! is_numeric( $longitude ) ) {
						return new \WP_Error( 'bad_request', 'Invalid "longitude" parameter.', array( 'status' => 400 ) );
					}

					$onlinemap_locations_repository = new OnlineMapLocations();
					$location = new Location(
						id: new LocationId( $location_id ),
						icon_id: new IconId( (int) $icon_id ),
						position: new Position( (float) $latitude, (float) $longitude ),
						address: new Address(
							country: new Country( $request->get_param( 'country' ) ?? '' ),
							state: new State( $request->get_param( 'state' ) ?? '' ),
							city: new City( $request->get_param( 'city' ) ?? '' ),
							postal: new Postal( $request->get_param( 'postal' ) ?? '' ),
							street: new Street( $request->get_param( 'street' ) ?? '' ),
							house: new House( $request->get_param( 'house' ) ?? '' ),
						)
					);
					$onlinemap_locations_repository->update( $location );

					$response = array(
						'id'       => $id,
						'icon_id'  => $icon_id,
						'position' => array(
							'latitude'  => $latitude,
							'longitude' => $longitude,
						),
					);

					return rest_ensure_response( $response );
				},
				'permission_callback' => '__return_true',
			)
		);
	}
}
